@@ -1,8 +1,8 @@
 /*
   Hatari - change.c
 
-  This file is distributed under the GNU Public License, version 2 or at
-  your option any later version. Read the file gpl.txt for details.
+  This file is distributed under the GNU General Public License, version 2
+  or at your option any later version. Read the file gpl.txt for details.
 
   This code handles run-time configuration changes. We keep all our
   configuration details in a structure called 'ConfigureParams'.  Before
@@ -103,16 +103,22 @@ bool Change_DoNeedReset(CNF_PARAMS *current, CNF_PARAMS *changed)
 	/* Did change machine type? */
 	if (changed->System.nMachineType != current->System.nMachineType)
 		return true;
+	/* did change ST Blitter? */
+	else if (current->System.nMachineType == MACHINE_ST &&
+		 current->System.bBlitter != changed->System.bBlitter)
+		return true;
 
 #if ENABLE_DSP_EMU
 	/* enabling DSP needs reset (disabling it not) */
 	if (current->System.nDSPType != DSP_TYPE_EMU &&
 	    changed->System.nDSPType == DSP_TYPE_EMU)
-	{
 		return true;
-	}
 #endif
 
+	/* did change CPU type? */
+	if (changed->System.nCpuLevel != current->System.nCpuLevel)
+		return true;
+
 #if ENABLE_WINUAE_CPU
 	/* Did change CPU address mode? */
 	if (changed->System.bAddressSpace24 != current->System.bAddressSpace24)
@@ -139,6 +145,10 @@ bool Change_DoNeedReset(CNF_PARAMS *current, CNF_PARAMS *changed)
 	if (current->Memory.nMemorySize != changed->Memory.nMemorySize)
 		return true;
 
+	/* MIDI related IRQs start/stop needs reset */
+	if (current->Midi.bEnableMidi != changed->Midi.bEnableMidi)
+		return true;
+
 	return false;
 }
 
