@@ -1,8 +1,8 @@
 /*
   Hatari - control.c
 
-  This file is distributed under the GNU Public License, version 2 or at
-  your option any later version. Read the file gpl.txt for details.
+  This file is distributed under the GNU General Public License, version 2
+  or at your option any later version. Read the file gpl.txt for details.
 
   This code processes commands from the Hatari control socket
 */
@@ -316,7 +316,7 @@ void Control_ProcessBuffer(char *buffer)
 			if (strcmp(cmd, "hatari-option") == 0) {
 				ok = Change_ApplyCommandline(arg);
 			} else if (strcmp(cmd, "hatari-debug") == 0) {
-				ok = DebugUI_RemoteParse(arg);
+				ok = DebugUI_ParseLine(arg);
 			} else if (strcmp(cmd, "hatari-shortcut") == 0) {
 				ok = Shortcut_Invoke(arg);
 			} else if (strcmp(cmd, "hatari-event") == 0) {
