@@ -1,8 +1,8 @@
 /*
   Hatari - cycles.c
 
-  This file is distributed under the GNU Public License, version 2 or at
-  your option any later version. Read the file gpl.txt for details.
+  This file is distributed under the GNU General Public License, version 2
+  or at your option any later version. Read the file gpl.txt for details.
 
   Here we take care of cycle counters. For performance reasons we don't increase
   all counters after each 68k instruction, but only one main counter.
@@ -23,6 +23,8 @@
 /* 2011/03/26	[NP]	In Cycles_GetCounterOnReadAccess, add a special case for opcode	*/
 /*			$11f8 'move.b xxx.w,xxx.w' (fix MOVE.B $ffff8209.w,$26.w in	*/
 /*			'Bird Mad Girl Show' demo's loader/protection)			*/
+/* 2012/08/19	[NP]	Add a global counter CyclesGlobalClockCounter to count cycles	*/
+/*			since the last reset.						*/
 
 
 const char Cycles_fileid[] = "Hatari cycles.c : " __DATE__ " " __TIME__;
@@ -33,13 +35,20 @@ const char Cycles_fileid[] = "Hatari cycles.c : " __DATE__ " " __TIME__;
 #include "cycles.h"
 
 
-int nCyclesMainCounter;				/* Main cycles counter */
+int	nCyclesMainCounter;			/* Main cycles counter since previous Cycles_UpdateCounters() */
 
 static int nCyclesCounter[CYCLES_COUNTER_MAX];	/* Array with all counters */
 
-int CurrentInstrCycles;
-int MovepByteNbr;				/* Number of the byte currently transferred in a movep (1..2 or 1..4) */
+Uint64	CyclesGlobalClockCounter = 0;		/* Global clock counter since starting Hatari (it's never reset afterwards) */
 
+int	CurrentInstrCycles;
+int	MovepByteNbr = 0;			/* Number of the byte currently transferred in a movep (1..2 or 1..4) */
+						/* 0 means current instruction is not a movep */
+
+
+static void	Cycles_UpdateCounters(void);
+static int	Cycles_GetInternalCycleOnReadAccess(void);
+static int	Cycles_GetInternalCycleOnWriteAccess(void);
 
 
 
@@ -52,6 +61,7 @@ void Cycles_MemorySnapShot_Capture(bool bSave)
 	/* Save/Restore details */
 	MemorySnapShot_Store(&nCyclesMainCounter, sizeof(nCyclesMainCounter));
 	MemorySnapShot_Store(nCyclesCounter, sizeof(nCyclesCounter));
+	MemorySnapShot_Store(&CyclesGlobalClockCounter, sizeof(CyclesGlobalClockCounter));
 	MemorySnapShot_Store(&CurrentInstrCycles, sizeof(CurrentInstrCycles));
 }
 
@@ -102,20 +112,18 @@ int Cycles_GetCounter(int nId)
 
 /*-----------------------------------------------------------------------*/
 /**
- * Read a counter on CPU memory read access by taking care of the instruction
- * type (add the needed amount of additional cycles).
+ * Compute the cycles where a read actually happens inside a specific
+ * instruction type. We use some common cases, this should be handled more
+ * accurately in the cpu emulation for each opcode.
  */
-int Cycles_GetCounterOnReadAccess(int nId)
+static int Cycles_GetInternalCycleOnReadAccess(void)
 {
-	int nAddCycles;
+	int AddCycles;
 	int Opcode;
 
-	/* Update counters first so we read an up-to-date value */
-	Cycles_UpdateCounters();
-
 	if ( BusMode == BUS_MODE_BLITTER )
 	{
-		nAddCycles = 4 + nWaitStateCycles;
+		AddCycles = 4 + nWaitStateCycles;
 	}
 	else							/* BUS_MODE_CPU */
 	{
@@ -126,39 +134,38 @@ int Cycles_GetCounterOnReadAccess(int nId)
 
 		/* Assume we use 'move src,dst' : access cycle depends on dst mode */
 		if ( Opcode == 0x11f8 )				/* move.b xxx.w,xxx.w (eg MOVE.B $ffff8209.w,$26.w in Bird Mad Girl Show) */
-			nAddCycles = CurrentInstrCycles + nWaitStateCycles - 8;		/* read is effective before the 8 write cycles for dst */
+			AddCycles = CurrentInstrCycles + nWaitStateCycles - 8;		/* read is effective before the 8 write cycles for dst */
 		else if ( OpcodeFamily == i_MVPRM )					/* eg movep.l d0,$ffc3(a1) in E605 (STE) */
-			nAddCycles = 12 + MovepByteNbr * 4;				/* [NP] FIXME, it works with E605 but gives 20-32 cycles instead of 16-28 */
+			AddCycles = 12 + MovepByteNbr * 4;				/* [NP] FIXME, it works with E605 but gives 20-32 cycles instead of 16-28 */
 											/* something must be wrong in video.c */
 		else
-			nAddCycles = CurrentInstrCycles + nWaitStateCycles;		/* assume dest is reg : read is effective at the end of the instr */
+			AddCycles = CurrentInstrCycles + nWaitStateCycles;		/* assume dest is reg : read is effective at the end of the instr */
 	}
 
-	return nCyclesCounter[nId] + nAddCycles;
+	return AddCycles;
 }
 
 
+
 /*-----------------------------------------------------------------------*/
 /**
- * Read a counter on CPU memory write access by taking care of the instruction
- * type (add the needed amount of additional cycles).
+ * Compute the cycles where a write actually happens inside a specific
+ * instruction type. We use some common cases, this should be handled more
+ * accurately in the cpu emulation for each opcode.
  */
-int Cycles_GetCounterOnWriteAccess(int nId)
+static int Cycles_GetInternalCycleOnWriteAccess(void)
 {
-	int nAddCycles;
-
-	/* Update counters first so we read an up-to-date value */
-	Cycles_UpdateCounters();
+	int AddCycles;
 
 	if ( BusMode == BUS_MODE_BLITTER )
 	{
-		nAddCycles = 4 + nWaitStateCycles;
+		AddCycles = 4 + nWaitStateCycles;
 	}
 	else							/* BUS_MODE_CPU */
 	{
 		/* TODO: Find proper cycles count depending on the type of the current instruction */
 		/* (e.g. movem is not correctly handled) */
-		nAddCycles = CurrentInstrCycles + nWaitStateCycles;
+		AddCycles = CurrentInstrCycles + nWaitStateCycles;
 
 		if ( OpcodeFamily == i_CLR )				/* should also be the case for add, sub, and, or, eor, neg, not */
 			;						/* Do nothing, the write is done during the last 4 cycles */
@@ -171,10 +178,73 @@ int Cycles_GetCounterOnWriteAccess(int nId)
 		{
 			/* assume the behaviour of a 'move' (since this is the most */
 			/* common instr used when requiring cycle precise writes) */
-			if ( nAddCycles >= 8 )
-				nAddCycles -= 4;			/* last 4 cycles are for prefetch */
+			if ( AddCycles >= 8 )
+				AddCycles -= 4;			/* last 4 cycles are for prefetch */
 		}
 	}
 
-	return nCyclesCounter[nId] + nAddCycles;
+	return AddCycles;
 }
+
+
+/*-----------------------------------------------------------------------*/
+/**
+ * Read a counter on CPU memory read access by taking care of the instruction
+ * type (add the needed amount of additional cycles).
+ */
+int Cycles_GetCounterOnReadAccess(int nId)
+{
+	int AddCycles;
+
+	AddCycles = Cycles_GetInternalCycleOnReadAccess();
+
+	return Cycles_GetCounter(nId) + AddCycles;
+}
+
+
+/*-----------------------------------------------------------------------*/
+/**
+ * Read a counter on CPU memory write access by taking care of the instruction
+ * type (add the needed amount of additional cycles).
+ */
+int Cycles_GetCounterOnWriteAccess(int nId)
+{
+	int AddCycles;
+
+	AddCycles = Cycles_GetInternalCycleOnWriteAccess();
+
+	return Cycles_GetCounter(nId) + AddCycles;
+}
+
+
+/*-----------------------------------------------------------------------*/
+/**
+ * Read the main clock counter on CPU memory read access by taking care of the instruction
+ * type (add the needed amount of additional cycles).
+ */
+Uint64 Cycles_GetClockCounterOnReadAccess(void)
+{
+	int AddCycles;
+
+	AddCycles = Cycles_GetInternalCycleOnReadAccess();
+
+	return CyclesGlobalClockCounter + AddCycles;
+}
+
+
+/*-----------------------------------------------------------------------*/
+/**
+ * Read the main clock counter on CPU memory write access by taking care of the instruction
+ * type (add the needed amount of additional cycles).
+ */
+Uint64 Cycles_GetClockCounterOnWriteAccess(void)
+{
+	int AddCycles;
+
+	AddCycles = Cycles_GetInternalCycleOnWriteAccess();
+
+	return CyclesGlobalClockCounter + AddCycles;
+}
+
+
+
