@@ -1,8 +1,8 @@
 /*
   Hatari - debugInfo.h
 
-  This file is distributed under the GNU Public License, version 2 or at
-  your option any later version. Read the file gpl.txt for details.
+  This file is distributed under the GNU General Public License, version 2
+  or at your option any later version. Read the file gpl.txt for details.
 
   Public atari components debugging infos header.
 */
@@ -10,8 +10,9 @@
 #ifndef HATARI_DEBUGINFO_H
 #define HATARI_DEBUGINFO_H
 
-/* for breakcond.c */
+/* for breakcond.c & profile.c */
 extern Uint32 DebugInfo_GetTEXT(void);
+extern Uint32 DebugInfo_GetTEXTEnd(void);
 extern Uint32 DebugInfo_GetDATA(void);
 extern Uint32 DebugInfo_GetBSS(void);
 
