@@ -1,8 +1,8 @@
 /*
   Hatari - debug.h
 
-  This file is distributed under the GNU Public License, version 2 or at
-  your option any later version. Read the file gpl.txt for details.
+  This file is distributed under the GNU General Public License, version 2
+  or at your option any later version. Read the file gpl.txt for details.
 
   Internal header used by debugger files.
 */
@@ -29,6 +29,8 @@ extern void DebugUI_PrintCmdHelp(const char *psCmd);
 extern int DebugCpu_Init(const dbgcommand_t **table);
 extern void DebugCpu_InitSession(void);
 
+extern bool DebugUI_ParseFile(const char *path, bool reinit);
+
 #ifdef ENABLE_DSP_EMU
 extern int DebugDsp_Init(const dbgcommand_t **table);
 extern void DebugDsp_InitSession(void);
