@@ -1,8 +1,8 @@
 /*
   Hatari - file.c
 
-  This file is distributed under the GNU Public License, version 2 or at
-  your option any later version. Read the file gpl.txt for details.
+  This file is distributed under the GNU General Public License, version 2
+  or at your option any later version. Read the file gpl.txt for details.
 
   Common file access functions.
 */
@@ -184,7 +184,12 @@ Uint8 *File_Read(const char *pszFileName, long *pFileSize, const char * const pp
 			do
 			{
 				/* Seek through the file until we hit the end... */
-				gzseek(hGzFile, 1024, SEEK_CUR);
+				char tmp[1024];
+				if (gzread(hGzFile, tmp, sizeof(tmp)) < 0)
+				{
+					fprintf(stderr, "Failed to read gzip file!\n");
+					return NULL;
+				}
 			}
 			while (!gzeof(hGzFile));
 			FileSize = gztell(hGzFile);
