@@ -26,7 +26,8 @@
 - (IBAction)coldReset:(id)sender;
 - (IBAction)insertDiskA:(id)sender;
 - (IBAction)insertDiskB:(id)sender;
-//- (IBAction)help:(id)sender;
+- (IBAction)help:(id)sender;
+- (IBAction)compat:(id)sender;
 - (IBAction)captureScreen:(id)sender;
 - (IBAction)captureAnimation:(id)sender;
 - (IBAction)endCaptureAnimation:(id)sender;
