@@ -1,8 +1,8 @@
 /*
   Hatari - hdc.c
 
-  This file is distributed under the GNU Public License, version 2 or at
-  your option any later version. Read the file gpl.txt for details.
+  This file is distributed under the GNU General Public License, version 2
+  or at your option any later version. Read the file gpl.txt for details.
 
   Low-level hard drive emulation
 */
@@ -32,7 +32,7 @@ const char HDC_fileid[] = "Hatari hdc.c : " __DATE__ " " __TIME__;
   operation interrupts the current operation. The DRQ status can
   be polled non-destructively in GPIP.
 
-  (For simplicity, the operation is finished immediatly,
+  (For simplicity, the operation is finished immediately,
   this is a potential bug, but I doubt it is significant,
   we just appear to have a very fast hard drive.)
 
@@ -497,7 +497,7 @@ static void HDC_Cmd_ReadSector(void)
 
 #ifdef HDC_VERBOSE
 	fprintf(stderr,"Reading %i sectors from 0x%x to addr: 0x%x\n",
-	        HDC_GetCount(), nLastBlockAddr, FDC_ReadDMAAddress());
+	        HDC_GetCount(), nLastBlockAddr, FDC_GetDMAAddress());
 #endif
 
 	/* seek to the position */
@@ -630,7 +630,7 @@ static void HDC_EmulateCommandPacket(void)
 	}
 
 	/* Update the led each time a command is processed */
-	Statusbar_EnableHDLed();
+	Statusbar_EnableHDLed( LED_STATE_ON );
 }
 
 
@@ -859,12 +859,14 @@ short int HDC_GetSectorCount(void)
  */
 void HDC_WriteCommandPacket(void)
 {
+	unsigned char b;
+
 	/* is HDC emulation enabled? */
 	if (!bAcsiEmuOn)
 		return;
 
 	/* command byte sent */
-	unsigned char b = IoMem_ReadByte(0xff8605);
+	b = IoMem_ReadByte(0xff8605);
 
 	/* Extract target and extended mode early, read acsi opcode */
 	if (HDCCommand.readCount == 0)
