@@ -1,8 +1,8 @@
 /*
   Hatari - gemdos.h
 
-  This file is distributed under the GNU Public License, version 2 or at
-  your option any later version. Read the file gpl.txt for details.
+  This file is distributed under the GNU General Public License, version 2
+  or at your option any later version. Read the file gpl.txt for details.
 */
 #ifndef HATARI_GEMDOS_H
 #define HATARI_GEMDOS_H
@@ -25,6 +25,7 @@ extern void GemDOS_InitDrives(void);
 extern void GemDOS_UnInitDrives(void);
 extern void GemDOS_MemorySnapShot_Capture(bool bSave);
 extern void GemDOS_CreateHardDriveFileName(int Drive, const char *pszFileName, char *pszDestName, int nDestNameLen);
+extern const char *GemDOS_GetLastProgramPath(void);
 extern void GemDOS_Info(Uint32 bShowOpcodes);
 extern void GemDOS_OpCode(void);
 extern void GemDOS_Boot(void);
