@@ -1,8 +1,8 @@
 /*
   Hatari - resolution.h
 
-  This file is distributed under the GNU Public License, version 2 or at
-  your option any later version. Read the file gpl.txt for details.
+  This file is distributed under the GNU General Public License, version 2
+  or at your option any later version. Read the file gpl.txt for details.
 */
 
 #ifndef HATARI_RESOLUTION_H
@@ -10,7 +10,7 @@
 
 extern void Resolution_Init(void);
 extern void Resolution_GetDesktopSize(int *width, int *height);
-extern void Resolution_GetLimits(int *width, int *height, int *bpp);
+extern void Resolution_GetLimits(int *width, int *height, int *bpp, bool keepDesktop);
 extern void Resolution_Search(int *width, int *height, int *bpp);
 
 #endif
