@@ -1,8 +1,8 @@
 /*
   Hatari - main.c
 
-  This file is distributed under the GNU Public License, version 2 or at
-  your option any later version. Read the file gpl.txt for details.
+  This file is distributed under the GNU General Public License, version 2
+  or at your option any later version. Read the file gpl.txt for details.
 
   Main initialization and event handling routines.
 */
@@ -13,6 +13,7 @@ const char Main_fileid[] = "Hatari main.c : " __DATE__ " " __TIME__;
 #include <SDL.h>
 
 #include "main.h"
+#include "version.h"
 #include "configuration.h"
 #include "control.h"
 #include "options.h"
@@ -23,6 +24,7 @@ const char Main_fileid[] = "Hatari main.c : " __DATE__ " " __TIME__;
 #include "gemdos.h"
 #include "hdc.h"
 #include "ide.h"
+#include "acia.h"
 #include "ikbd.h"
 #include "ioMem.h"
 #include "keymap.h"
@@ -591,11 +593,14 @@ static void Main_Init(void)
 	Screen_Init();
 	Main_SetTitle(NULL);
 	HostScreen_Init();
+
+	ACIA_Init( ACIA_Array , MachineClocks.ACIA_Freq , MachineClocks.ACIA_Freq );
+	IKBD_Init();			/* After ACIA_Init */
+
 	DSP_Init();
 	Floppy_Init();
 	M68000_Init();                /* Init CPU emulation */
 	Audio_Init();
-	DmaSnd_Init();
 	Keymap_Init();
 
 	/* Init HD emulation */
@@ -771,7 +776,7 @@ int main(int argc, char *argv[])
 	/* Check if SDL_Delay is accurate */
 	Main_CheckForAccurateDelays();
 
-	if ( AviRecordOnStartup )	/* Immediatly starts avi recording ? */
+	if ( AviRecordOnStartup )	/* Immediately starts avi recording ? */
 		Avi_StartRecording ( ConfigureParams.Video.AviRecordFile , ConfigureParams.Screen.bCrop ,
 			ConfigureParams.Video.AviRecordFps == 0 ?
 				ClocksTimings_GetVBLPerSec ( ConfigureParams.System.nMachineType , nScreenRefreshRate ) :
