@@ -1,8 +1,8 @@
 /*
   Hatari - memorySnapShot.c
 
-  This file is distributed under the GNU Public License, version 2 or at
-  your option any later version. Read the file gpl.txt for details.
+  This file is distributed under the GNU General Public License, version 2
+  or at your option any later version. Read the file gpl.txt for details.
 
   Memory Snapshot
 
@@ -30,6 +30,7 @@ const char MemorySnapShot_fileid[] = "Hatari memorySnapShot.c : " __DATE__ " " _
 #include "file.h"
 #include "floppy.h"
 #include "gemdos.h"
+#include "acia.h"
 #include "ikbd.h"
 #include "cycInt.h"
 #include "cycles.h"
@@ -52,7 +53,7 @@ const char MemorySnapShot_fileid[] = "Hatari memorySnapShot.c : " __DATE__ " " _
 #include "statusbar.h"
 
 
-#define VERSION_STRING      "1.6.2"   /* Version number of compatible memory snapshots - Always 6 bytes (inc' NULL) */
+#define VERSION_STRING      "1.7.0"   /* Version number of compatible memory snapshots - Always 6 bytes (inc' NULL) */
 
 #define COMPRESS_MEMORYSNAPSHOT       /* Compress snapshots to reduce disk space used */
 
@@ -244,6 +245,7 @@ void MemorySnapShot_Capture(const char *pszFileName, bool bConfirm)
 		FDC_MemorySnapShot_Capture(true);
 		Floppy_MemorySnapShot_Capture(true);
 		GemDOS_MemorySnapShot_Capture(true);
+		ACIA_MemorySnapShot_Capture(true);
 		IKBD_MemorySnapShot_Capture(true);
 		CycInt_MemorySnapShot_Capture(true);
 		Cycles_MemorySnapShot_Capture(true);
@@ -296,7 +298,8 @@ void MemorySnapShot_Restore(const char *pszFileName, bool bConfirm)
 		FDC_MemorySnapShot_Capture(false);
 		Floppy_MemorySnapShot_Capture(false);
 		GemDOS_MemorySnapShot_Capture(false);
-		IKBD_MemorySnapShot_Capture(false);
+		ACIA_MemorySnapShot_Capture(false);
+		IKBD_MemorySnapShot_Capture(false);			/* After ACIA */
 		CycInt_MemorySnapShot_Capture(false);
 		Cycles_MemorySnapShot_Capture(false);
 		M68000_MemorySnapShot_Capture(false);
